proc periodic-dump {} {
    global Prefs

    if {$Prefs(autodump)} {
	dump_text
	after [expr 1000 * $Prefs(autodumpInterval)] periodic-dump
    }
}



proc dump_text {} {
    set dir [resource htmlDir]
    set file "$dir/index.html"
    set chan [open $file "w"]
    puts "writing to $file"
    dump_text_to $chan false

    close $chan

    set file "$dir/index-compact.html"
    set chan [open $file "w"]
    puts "writing to $file"
    dump_text_to $chan true
    
    close $chan
}

proc dump_text_to { chan compact } {
    global State names stats



    set now [clock format [clock seconds] -format "%a %D %I:%M:%S %p"]

    puts $chan "<TITLE>Rtpmon dump $now</title>"
    puts $chan "<h1>Rtpmon dump $now</h1>"

    if {$compact} {
	set colwidth ""
	puts $chan "<a href=\"index.html\">Full display</a><p>"
    } else {
	set colwidth "width=\"40\""
	puts $chan "<a href=\"index-compact.html\">Compact display</a><p>"
    }
    
    set group [resource defaultHostSpec]
    puts $chan "Monitor target: <b>$group</b><p>"
    set nsend [llength $State(senders)]
    set nlisten [llength $State(listeners)]
    puts $chan "$nsend senders<p>" 
    puts $chan "$nlisten listeners<p>" 

    puts $chan "<table border=\"1\">"

    puts $chan "<tr>"
    
    puts $chan "<th></th>"
    set snum 0
    foreach s $State(senders) {
	set name $names($s)
	if [regsub {^([^@]*)@([^@]*)} $name "\\1 @ \\2" foo] {
	    set name $foo
	    if [ regsub -all {\.} $name ". " newname ] {
		set name $newname
	    }
	}

	if {$compact} {
	    set name "S$snum"
	    set legend($snum) $names($s)
	    incr snum
	}

	set url [create_info_page $s]
	puts $chan "<th $colwidth><a href=\"$url\">$name</a></th>"
    }
    puts $chan "</tr>"


    foreach l $State(listeners) {
	if [info exists cname_cache($l)] {
	    set lcname $cname_cache($l)
	} else {
	    set lcname [lindex [lindex [$l info] 0] 1]
	    set cname_cache($l) $lcname
	}
	puts $chan "<tr>"
	set name $names($l)
	if [regsub {^([^@]*)@([^@]*)} $name "\\1 @ \\2" foo] {
	    set name $foo
	}
	set url [create_info_page $l]
	puts $chan "<th width=\"0\" align=\"left\"><a href=\"$url\">$name</a></th>"
	
        foreach s $State(senders) {
	    set idx "${l}:${s}"
	    
	    set bg ""
	    set val ""
	    if [info exists stats($idx)] {
		set val $stats($idx)
		if [regexp {([0-9]+) *%}  $val foo pct] {
		    if {$pct < 10 } {
			set color "green"
		    } elseif {$pct > 70} {
			set color "red"
		    } else {
			set color "yellow"
		    }
		    set bg " bgcolor=\"$color\""
		} else {
		    if [info exists cname_cache($s)] {
			set scname $cname_cache($s)
		    } else {
			set scname [lindex [lindex [$s info] 0] 1]
			set cname_cache($s) $scname
		    }

#		    puts "$scname $lcname"
		    if { $scname == $lcname } {
#			puts "got match"
			set bg " bgcolor=\"black\""
			set val "."
		    }
		}
	    }


	    puts $chan "<td $colwidth align=\"center\"$bg>$val</td>"
	}
	puts $chan "</tr>"
    }
    puts $chan "</table>"

    if {$compact} {
	puts $chan "<p>"
	puts $chan "Sender legend:<p>"
	puts $chan "<table>"

	set half [expr int(ceil($snum / 2.0))]
	for  { set i 0 } { $i < $half  } {incr i } {
	    puts $chan "<tr>"
	    puts $chan "<td>S$i</td>"
	    puts $chan "<td>$legend($i)</td>"

	    set i2 [expr $i + $half]
	    if { $i2 < $snum } {
		puts $chan "<td>S$i2</td>"
		puts $chan "<td>$legend($i2)</td>"
	    }
	    puts $chan "</tr>"
	}
	puts $chan "</table>"
    }
}

proc create_info_page s {
    set dir [resource htmlDir]
    set base "info_$s.html"
    set file "$dir/$base"
    set chan [open $file "w"]

    set name [$s name]

    puts $chan "<title>$name</title>"
    puts $chan "<h1>$name</h1>"

    set keys [$s info]

    foreach k $keys {
	set label [lindex $k 0]
	set value [lindex $k 1]

	if {$label == "name"} {
	    set name $value
	} elseif {$label == "cname"} {
	    set cname $value
	}
    }

    puts $chan "<b>$name</b><br>"
    puts $chan "<b>$cname</b><p>"

    puts $chan "<hr>"

    puts $chan "<table>"
    foreach k $keys {
	set label [lindex $k 0]
	set value [lindex $k 1]

	puts $chan "<tr>"
	puts $chan "<th align=\"left\">$label</th> <td align=\"left\">$value</td>"
	puts $chan "</tr>"
    }
    puts $chan "</table>"

    close $chan
    return $base
}
